# $Id: 00-compile.t 115 2008-12-01 21:17:51Z swistow $

use strict;
use Test::More tests => 4;

use_ok('XML::Feed');
use_ok('XML::Feed::Entry');
use_ok('XML::Feed::Format::RSS');
use_ok('XML::Feed::Format::Atom');
